// FILE: ./assets/js/utilities/Utilities.js
function Utilities(ScreenService, MessageService, General, ZhPreferences, Query, validateEngine, ZhFilterPreferences, templateManager) {

/*global Util, Configuration, $*/

    this.changeTagRichText = function(fieldValue) {
        if (typeof(fieldValue) === 'string') {
            var newFieldValue;

            if (fieldValue.substr(0, 3) == '<p>') {
                newFieldValue = fieldValue.replace(/<p>/g, '').replace(/<\/p>/g, '<br>');
            }
            else if (fieldValue.substr(0, 3) == '<p ') {
                newFieldValue = fieldValue.replace(/<p/g, '<span').replace(/<\/p>/g, '</span><br>');
            }
            else {
                newFieldValue = fieldValue;
            }

            if (fieldValue.substr(-4) == '</p>') {
                newFieldValue = newFieldValue.slice('', -4);
            }

            return newFieldValue;
        }
        else {
            return fieldValue;
        }
    };

    this.startFilter = function(widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };

                    currentRow = filter.FILTER_ROW;

                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            }
            else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        return getFilterRowWithDefaultValues(widget.fields);
    };

    var getFilterRowWithDefaultValues = function(filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };
    
    function formatStringToFloat(word, simbolThousands, simbolDecimals){
        word = word.split(simbolThousands).join('');
        word = word.replace(simbolDecimals, '.');    
        return word;
    }


    function verifyRange(value, validationAttrs) {
        const maxValue = validationAttrs.max || '9999999.99';
        const minValue = validationAttrs.min || 0;
        const simbolThousands = validationAttrs.thousands || '.';
        const simbolDecimals = validationAttrs.decimals || ',';
        let valid = true;
        let msg = validationAttrs.message || (MessageService.getMessageByCode("sup#NAO_ESTA_INTERVALO") + minValue + " a " + maxValue);
        
        if (typeof(value) == 'string') {
            value = formatStringToFloat(value, simbolThousands, simbolDecimals);
        }

        if (value && (parseFloat(value) > parseFloat(maxValue) || parseFloat(value) < parseFloat(minValue))) {
            valid = false;
        }
        
        return  {"valid": valid, "message": msg};
    }
    
    function validaProduto(value, validationAttrs) {

        var filtro = templateManager.container.getActiveWidget();
        var retorno = {
            "valid": true
        };
        let field = validationAttrs.field;
        let fieldProduto;
        const final = filtro.currentRow.CDPRODFIN;
        const inicial = filtro.currentRow.CDPRODINI;

        if (value) {
            fieldProduto = field == "NMPRODFIN" ? inicial : final;
            if (fieldProduto) {
                if (inicial > final) {
                    return {
                        "valid": false,
                        "message": validationAttrs.message || MessageService.getMessageByCode("sup#PRODUTO_INICIAL_MENOR_IGUAL_FINAL")
                    };
                }
            }
        }
        else if (field == "NMPRODFIN" && filtro.currentRow.CDPRODINI || field == "NMPRODINI" && filtro.currentRow.CDPRODFIN) {
            let msg = field == "NMPRODFIN" ? 'Produto final obrigatório' : 'Produto inicial obrigatório';
            return {
                "valid": false,
                "message": validationAttrs.message || ScreenService.i18n(msg)
            };
        }

        return retorno;
    }
    
    var registeredValidations = validateEngine.getMappedValidations();
    
    if (!registeredValidations.hasOwnProperty('verifyRange')) {
        validateEngine.registerValidation('verifyRange', verifyRange);
    }
    
    
    if (!registeredValidations.hasOwnProperty('validaProduto')) {
        validateEngine.registerValidation('validaProduto', validaProduto);
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Utilities', Utilities);
});


// FILE: ./assets/js/controllers/acompanhamento_aprovacao.js
function acompanhamento_aprovacao(ScreenService, templateManager, General, $timeout, Utilities) {
    var timelines, eventsMinDistance, self = this;

    this.load = function() {
        let templateUrl = {
            package: "teste",
            baseUrl: "./assets/templates/html/acompanhamento.html"
        };
        window.templateUrls.push(templateUrl);

        templateUrl = {
            package: "textTemplate",
            baseUrl: "./assets/templates/html/textTemplate.html"
        };
        window.templateUrls.push(templateUrl);
    };

    this.cancelar = function(widget) {
        ScreenService.closePopup();
        widget.activate();
    };
    
    this.checkFilterBeforeConfirm = function(args) {
        const filterRow = args.e.preference.FILTER_ROW;
        
        const widgetAlcadas = templateManager.container.getWidget('ALCADAS');
        
        let P_TDTPROGENTAF = '';
        let P_TDTINCLUSAO = '';
        
        
        if(filterRow.DTINCLUSAO.start == '' && filterRow.DTINCLUSAO.end == ''){
            P_TDTINCLUSAO = 'T';
        }
        if(filterRow.DTPROGENTAF.start == '' && filterRow.DTPROGENTAF.end == ''){
            P_TDTPROGENTAF = 'T';
        }
        
        widgetAlcadas.dataSourceFilter = [
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value":filterRow.CDFILDEST && filterRow.CDFILDEST.length  ? filterRow.CDFILDEST.join('_') : 'T'
            },
            {
                "name": "NMALCADA",
                "operator": "=",
                "value": filterRow.NMALCADA && filterRow.NMALCADA.length  ? filterRow.NMALCADA.join('_') : 'T'
            },
            {
                "name": "IDCLASSIFIC",
                "operator": "LIKE",
                "value":filterRow.IDCLASSIFIC && filterRow.IDCLASSIFIC.length  ? filterRow.IDCLASSIFIC.join('_') : 'T'
            },
            {
                "name": "IDSITAFALCADA",
                "operator": "=",
                "value":filterRow.IDSITAFALCADA && filterRow.IDSITAFALCADA.length  ? filterRow.IDSITAFALCADA.join('_') : 'T'
            },
            {
                "name": "DTINCLUSAOINI",
                "operator": "=",
                "value":filterRow.DTINCLUSAO.start != '' ? filterRow.DTINCLUSAO.start : 'T'
            },
            {
                "name": "DTINCLUSAOFIN",
                "operator": "=",
                "value":filterRow.DTINCLUSAO.end != '' ? filterRow.DTINCLUSAO.end : 'T'
            },
            {
                "name": "DTPROGENTAFFIN",
                "operator": "=",
                "value": filterRow.DTPROGENTAF.end != '' ? filterRow.DTPROGENTAF.end : 'T'
            },
            {
                "name": "DTPROGENTAFINI",
                "operator": "=",
                "value": filterRow.DTPROGENTAF.start != '' ? filterRow.DTPROGENTAF.start : 'T'
            },
            {
                "name": "P_TDTPROGENTAF",
                "operator": "=",
                "value":P_TDTPROGENTAF
            },
            {
                "name": "P_TDTINCLUSAO",
                "operator": "=",
                "value": P_TDTINCLUSAO
            }
        ];
        
        if(widgetAlcadas.getField('DTPROGENTAF2')){
            widgetAlcadas.getField('DTPROGENTAF2').name = 'DTPROGENTAF';
        }
        //widgetAlcadas.dataSourceFilter = widgetAlcadas.dataSourceFilter.slice(0,10);
        

    };
    
    this.checkFilterAfterApply  = function(args) {
        const widget = templateManager.container.getWidget('ALCADAS');
        widget.reload();
    };
    
    this.setFilterWidgetItemInconsistente = function() {
        const itemInconsistente = templateManager.container.getWidget('ITEM_INCONSISTENTE');
        
        itemInconsistente.dataSourceFilter = [
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value":itemInconsistente.getParent().getParent().currentRow.CDFILDEST
            },
            {
                "name": "CDFILLANC",
                "operator": "=",
                "value":itemInconsistente.getParent().getParent().currentRow.CDFILLANC
            },
            {
                "name": "NRAUTOFORN",
                "operator": "=",
                "value":itemInconsistente.getParent().getParent().currentRow.NRDEMANDA
            }
        ];
        
        itemInconsistente.reload();
    };

    this.abrirDetalhes = function(widget) {
        const alcadasWidget = templateManager.container.getWidget("ALCADAS");
        const currentRowAlcadaWidget = alcadasWidget.currentRow;
        
        templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").dataSourceFilter = [
            {
                "name": "NRCONFIGALCADA",
                "operator": "=",
                "value": currentRowAlcadaWidget.NRCONFIGALCADA
            },
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value": currentRowAlcadaWidget.CDFILDEST
            },
            {
                "name": "NRDEMANDA",
                "operator": "=",
                "value": currentRowAlcadaWidget.NRDEMANDA
            },
            {
                "name": "CDFILLANC",
                "operator": "=",
                "value": currentRowAlcadaWidget.CDFILLANC
            }
        ];

        this.loadSwipe(templateManager.container.getWidget("ACOMPANHAMENTO"));
        templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").reload();
        ScreenService.openSwipe(alcadasWidget.container.getWidget("MAIN_TABBED"));
    };

    this.loadSwipe = function(owner) {
        General.callRestEngine("wkf#/timeline", "FilterData", owner.getField("TIMELINE").dataSourceFilter).then(function(response) {
            owner.getField("TIMELINE").dataSource.data = response.dataset.timeline;
            modificarField(owner);
        }.bind(this));
    };

    this.voltar = function() {
        ScreenService.closeSwipe();
        templateManager.container.getWidget('ALCADAS').activate();

        setTimeout(function() {
            const widgetAcompanhamento = templateManager.container.getWidget("ACOMPANHAMENTO");
            
            const arrayFields = [
                "IDAPROVADO",
                "NMOPERADOR",
                "DTAPROVACAO",
                "DTINCLUSAO",
                "VRAPROVCMP",
                "APROVPRINC",
                "DSJUSTIFICA"
            ];
            
            for(let field of arrayFields) {
                widgetAcompanhamento.getField(field).value('');
            }
        }.bind(this), 100);

    };

    function modificarField(widget) {
        const timelines = $('.cd-horizontal-timeline');
        eventsMinDistance = 60;

        if (document.getElementsByClassName("cd-horizontal-timeline").length > 0) {
            if (timelines.length > 0) {
                initTimeline(timelines);
            }
        } else {
            setTimeout(function() {
                modificarField(widget);
            }.bind(this), 100);
        }
    }

    function initTimeline(timelines) {
        timelines.each(function() {
            setTimeout(function() {
                let timeline = $(this),
                    timelineComponents = {};
                //cache timeline components 
                timelineComponents.timelineWrapper = timeline.find('.events-wrapper');
                timelineComponents.eventsWrapper = timelineComponents.timelineWrapper.children('.events');
                timelineComponents.fillingLine = timelineComponents.eventsWrapper.children('.filling-line');
                timelineComponents.timelineEvents = timelineComponents.eventsWrapper.find('.content-box');
                timelineComponents.timelineNavigation = timeline.find('.cd-timeline-navigation');

                let data = templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").dataSource.data,
                    situacao = "exclamation";

                //assign a width to the timeline
                let timelineTotWidth = setTimelineWidth(timelineComponents, eventsMinDistance);
                //the timeline has been initialize - show it
                timeline.addClass('loaded');

                for (let i = 0; i < data.length; i++) {
                    if ((data[i].SITUACAO == "exclamation") || (data[i].SITUACAO == "error")) {
                        timelineComponents.timelineEvents.eq(i).addClass('selected');
                        situacao = data[i].SITUACAO;
                    }
                }

                updateOlderEvents($(".selected"));
                updateFilling($(".selected"), timelineComponents.fillingLine, timelineTotWidth, situacao);
                translateTimeline(timelineComponents, timelineTotWidth, 0);

                if (data.length > 0) {
                    const widgetAcompanhamento = templateManager.container.getWidget("ACOMPANHAMENTO");
                    
                    widgetAcompanhamento.getField("IDAPROVADO").value(formataDetalhes(data[0].IDAPROVADO));
                    widgetAcompanhamento.getField("NMOPERADOR").value(formataDetalhes(data[0].NMOPERADOR));
                    formataDataAprovacao(templateManager.container.getWidget("ACOMPANHAMENTO").getField("DTAPROVACAO"), data[0].IDAPROVADO, data[0].DTAPROVACAO);
                    widgetAcompanhamento.getField("DTINCLUSAO").value(formataDetalhes(data[0].DTINCLUSAO));
                    
                    //widgetAcompanhamento.getField("DTPROGENTAF").value(formataDetalhes(data[0].DTPROGENTAF));
                    
                    widgetAcompanhamento.getField("VRAPROVCMP").value(formataDetalhes(data[0].VRAPROVCMP));
                    widgetAcompanhamento.getField("APROVPRINC").value(formataDetalhes(data[0].APROVPRINC));
                    widgetAcompanhamento.getField("DSJUSTIFICA").value(formataDetalhes(data[0].DSJUSTIFICA));
                    widgetAcompanhamento.getField("DSMOTVAFALCADA").value(formataDetalhes(templateManager.container.getWidget("ALCADAS").currentRow.DSMOTVAFALCADA));
                }
                
                //detect click on the next arrow
                timelineComponents.timelineNavigation.on('click', '.next', function(event) {
                    event.preventDefault();
                    updateSlide(timelineComponents, timelineTotWidth, 'next');
                });
                
                //detect click on the prev arrow
                timelineComponents.timelineNavigation.on('click', '.prev', function(event) {
                    event.preventDefault();
                    updateSlide(timelineComponents, timelineTotWidth, 'prev');
                });
                
                //detect click on the a single event - show new event content
                timelineComponents.eventsWrapper.on('click', 'a', function(event) {
                    event.preventDefault();

                    var data = templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").dataSource.data;

                    for (var i = 0; i < data.length; i++) {
                        if (data[i].NMOPERADOR == $(this).text()) {
                            const widgetAcompanhamento = templateManager.container.getWidget("ACOMPANHAMENTO");
                            
                            widgetAcompanhamento.getField("IDAPROVADO").value(formataDetalhes(data[i].IDAPROVADO));
                            widgetAcompanhamento.getField("NMOPERADOR").value(formataDetalhes(data[i].NMOPERADOR));
                            formataDataAprovacao(widgetAcompanhamento.getField("DTAPROVACAO"), data[i].IDAPROVADO, data[i].DTAPROVACAO);
                            widgetAcompanhamento.getField("DTINCLUSAO").value(formataDetalhes(data[i].DTINCLUSAO));
                            
                            //widgetAcompanhamento.getField("DTPROGENTAF").value(formataDetalhes(data[i].DTPROGENTAF));
                            
                            widgetAcompanhamento.getField("VRAPROVCMP").value(formataDetalhes(data[i].VRAPROVCMP));
                            widgetAcompanhamento.getField("APROVPRINC").value(formataDetalhes(data[i].APROVPRINC));
                            widgetAcompanhamento.getField("DSJUSTIFICA").value(formataDetalhes(data[i].DSJUSTIFICA));
                        }
                    }
                });

            }.bind(this), 200);

        });
    }
    
    function formataDataAprovacao(widget, situacao, value) {
        if (situacao == "RECUSADO") {
            widget.label = "Data de Reprovação";
        } else {
            widget.label = "Data de Aprovação";
        }


        widget.value(formataDetalhes(value));
    }

    function formataDetalhes(param) {
        if (param != null) {
            return param;
        }

        return "-";
    }

    function updateSlide(timelineComponents, timelineTotWidth, string) {
        //retrieve translateX value of timelineComponents.eventsWrapper
        const translateValue = getTranslateValue(timelineComponents.eventsWrapper),
            wrapperWidth = Number(timelineComponents.timelineWrapper.css('width').replace('px', ''));
        //translate the timeline to the left('next')/right('prev') 
        if (string == 'next') {
            translateTimeline(timelineComponents, translateValue - wrapperWidth + eventsMinDistance, wrapperWidth - timelineTotWidth);
        } else {
            translateTimeline(timelineComponents, translateValue + wrapperWidth - eventsMinDistance);
        }
    }

    function translateTimeline(timelineComponents, value, totWidth) {
        const eventsWrapper = timelineComponents.eventsWrapper.get(0);
        value = (value > 0) ? 0 : value; //only negative translate value
        value = ((typeof totWidth !== 'undefined') && value < totWidth) ? totWidth : value; //do not translate more than timeline width
        setTransformValue(eventsWrapper, 'translateX', value + 'px');
        //update navigation arrows visibility
        if (value == 0) {
            timelineComponents.timelineNavigation.find('.prev').addClass('inactive');
        } else { 
            timelineComponents.timelineNavigation.find('.prev').removeClass('inactive');
        }
        
        if (value == totWidth) {
            timelineComponents.timelineNavigation.find('.next').addClass('inactive');
        } else {
            timelineComponents.timelineNavigation.find('.next').removeClass('inactive');
        }
    }

    function updateFilling(selectedEvent, filling, totWidth, situacao) {
        //change .filling-line length according to the selected event
        const eventStyle = window.getComputedStyle(selectedEvent.get(0), null);
        
        if (eventStyle != null) {
            let eventLeft = eventStyle.getPropertyValue("left");
            const eventWidth = eventStyle.getPropertyValue("width");

            eventLeft = Number(eventLeft.replace('px', '')) + Number(eventWidth.replace('px', '')) / 2;
            
            const scaleValue = eventLeft / totWidth;
            setTransformValue(filling.get(0), 'scaleX', scaleValue, situacao);
        }
    }
    
    function setTimelineWidth(timelineComponents, width) {
        let i = 0;
        let offset = 0;
        
        const numItens = timelineComponents.timelineEvents.length;
        const widthCircles = timelineComponents.timelineNavigation.find('.next').width();
        const minSize = timelineComponents.timelineNavigation.find('.next').position().left - widthCircles;
        const totalWidth = numItens * 200;
        const widthContentBox = 150;

        if (totalWidth > minSize) {
            offset = 20;
            for (i = 0; i < numItens; i++) {
                timelineComponents.timelineEvents.eq(i).css('left', offset + 'px');
                offset += 200;
            }
        } else {
            var distance = ((minSize - (widthContentBox * numItens)) / (numItens + 1));

            offset = distance;
            for (i = 0; i < numItens; i++) {
                timelineComponents.timelineEvents.eq(i).css('left', offset + 'px');
                offset += distance + widthContentBox;
            }
        }

        if (totalWidth > minSize) {
            timelineComponents.eventsWrapper.css('width', totalWidth + 'px');
            return totalWidth;
        } else {
            timelineComponents.eventsWrapper.css('width', minSize + 'px');
            timelineComponents.timelineNavigation.find('.prev').addClass('inactive');
            timelineComponents.timelineNavigation.find('.next').addClass('inactive');
            return minSize;
        }
    }

    function updateOlderEvents(event) {
        event.parent('li').prevAll('li').children('a').addClass('older-event').end().end().nextAll('li').children('a').removeClass('older-event');
    }

    function getTranslateValue(timeline) {
        let timelineStyle = window.getComputedStyle(timeline.get(0), null),
            timelineTranslate = timelineStyle.getPropertyValue("-webkit-transform") ||
            timelineStyle.getPropertyValue("-moz-transform") ||
            timelineStyle.getPropertyValue("-ms-transform") ||
            timelineStyle.getPropertyValue("-o-transform") ||
            timelineStyle.getPropertyValue("transform");

        let translateValue;

        if (timelineTranslate.indexOf('(') >= 0) {
            timelineTranslate = timelineTranslate.split('(')[1];
            timelineTranslate = timelineTranslate.split(')')[0];
            timelineTranslate = timelineTranslate.split(',');
            translateValue = timelineTranslate[4];
        } else {
            translateValue = 0;
        }

        return Number(translateValue);
    }

    function setTransformValue(element, property, value, situacao) {

        if (situacao == "exclamation") {
            // gradient from green to yellow
            element.style.background = "linear-gradient(270deg,#E2B900,#179021 30%)";
        } else if (situacao == "error") {
            // gradient from green to red
            element.style.background = "linear-gradient(270deg,#E70000,#179021 30%)";
        } else {
            // all gray
            element.style.background = "linear-gradient(270deg,#6A6A6A,#6A6A6A 30%)";
        }

        element.style["-webkit-transform"] = property + "(" + value + ")";
        element.style["-moz-transform"] = property + "(" + value + ")";
        element.style["-ms-transform"] = property + "(" + value + ")";
        element.style["-o-transform"] = property + "(" + value + ")";
        element.style.transform = property + "(" + value + ")";
    }

    //based on http://stackoverflow.com/questions/542938/how-do-i-get-the-number-of-days-between-two-dates-in-javascript
    function parseDate(events) {
        const dateArrays = [];
        
        events.each(function() {
            let dateComp = $(this).data('date').split('/'),
                newDate = new Date(dateComp[2], dateComp[1] - 1, dateComp[0]);
            dateArrays.push(newDate);
        });
        
        return dateArrays;
    }

    function daydiff(first, second) {
        return Math.round((second - first));
    }

    function minLapse(dates) {
        //determine the minimum distance among events
        const  dateDistances = [];
        
        for (let i = 1; i < dates.length; i++) {
            let distance = daydiff(dates[i - 1], dates[i]);
            dateDistances.push(distance);
        }
        
        return Math.min.apply(null, dateDistances);
    }

    /*
    	How to tell if a DOM element is visible in the current viewport?
    	http://stackoverflow.com/questions/123999/how-to-tell-if-a-dom-element-is-visible-in-the-current-viewport
    */
    function elementInViewport(el) {
        let top = el.offsetTop;
        let left = el.offsetLeft;
        const width = el.offsetWidth;
        const height = el.offsetHeight;

        while (el.offsetParent) {
            el = el.offsetParent;
            top += el.offsetTop;
            left += el.offsetLeft;
        }

        return (
            top < (window.pageYOffset + window.innerHeight) &&
            left < (window.pageXOffset + window.innerWidth) &&
            (top + height) > window.pageYOffset &&
            (left + width) > window.pageXOffset
        );
    }

}

Configuration(function(ContextRegister) {
    ContextRegister.register('acompanhamento_aprovacao', acompanhamento_aprovacao);
});

